/**
* ReactPlugin.ts
* @copyright Microsoft 2019
*/
import { IAppInsights, IConfig, IEventTelemetry, IExceptionTelemetry, IMetricTelemetry, IPageViewTelemetry, ITraceTelemetry, ITelemetryContext as Common_ITelemetryContext } from "@microsoft/applicationinsights-common";
import { BaseTelemetryPlugin, IAppInsightsCore, IConfiguration, ICookieMgr, ICustomProperties, IPlugin, IProcessTelemetryContext, ITelemetryItem, ITelemetryPluginChain } from "@microsoft/applicationinsights-core-js";
export default class ReactPlugin extends BaseTelemetryPlugin {
    priority: number;
    identifier: string;
    readonly context: Common_ITelemetryContext;
    constructor();
    initialize(config: IConfiguration & IConfig, core: IAppInsightsCore, extensions: IPlugin[], pluginChain?: ITelemetryPluginChain): void;
    /**
     * Get the current cookie manager for this instance
     */
    getCookieMgr(): ICookieMgr;
    /**
     * Get application insights instance.
     */
    getAppInsights(): IAppInsights;
    /**
     * Add Part A fields to the event
     * @param event The event that needs to be processed
     */
    processTelemetry(event: ITelemetryItem, itemCtx?: IProcessTelemetryContext): void;
    trackMetric(metric: IMetricTelemetry, customProperties: ICustomProperties): void;
    trackPageView(pageView: IPageViewTelemetry): void;
    trackEvent(event: IEventTelemetry, customProperties?: ICustomProperties): void;
    trackException(exception: IExceptionTelemetry, customProperties?: ICustomProperties): void;
    trackTrace(trace: ITraceTelemetry, customProperties?: ICustomProperties): void;
}
