import * as React from 'react';
import ReactPlugin from './ReactPlugin';
/**
 * Higher-order component base class to hook Application Insights tracking
 * in a React component's lifecycle.
 */
export declare abstract class AITrackedComponentBase<P> extends React.Component<P> {
    protected _mountTimestamp: number;
    protected _firstActiveTimestamp: number;
    protected _idleStartTimestamp: number;
    protected _lastActiveTimestamp: number;
    protected _totalIdleTime: number;
    protected _idleCount: number;
    protected _idleTimeout: number;
    protected _intervalId?: any;
    protected _componentName: string;
    protected _reactPlugin: ReactPlugin;
    constructor(props: P, reactPlugin: ReactPlugin, componentName: string);
    componentDidMount(): void;
    componentWillUnmount(): void;
    protected trackActivity: (e: React.SyntheticEvent<any>) => void;
    private getEngagementTimeSeconds;
}
/**
 * Higher-order component function to hook Application Insights tracking
 * in a React component's lifecycle.
 *
 * @param reactPlugin ReactPlugin instance
 * @param Component the React component to be instrumented
 * @param componentName (optional) component name
 * @param className (optional) className of the HOC div
 */
export default function withAITracking<P>(reactPlugin: ReactPlugin, Component: React.ComponentType<P>, componentName?: string, className?: string): React.ComponentClass<P>;
